package nl.overheid.stelsel.digimelding.actions;

import nl.overheid.stelsel.digimelding.annotatie.EchoRequest;
import nl.overheid.stelsel.digimelding.clients.AnnotatieClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.net.URISyntaxException;
import java.util.Map;

/**
 * An action performing the 'EchoRequest'.
 */
@Component
public class EchoAction implements IAction {

    private AnnotatieClient annotatieClient;
    private ActionResult actionResult;

    @Autowired
    public EchoAction(AnnotatieClient annotatieClient) {
        this.annotatieClient = annotatieClient;
    }

    /**
     * Perform an EchoRequest. Set the response on the ActionResult.
     *
     * @param args - he arguments of the request (should contains a key for 'oin' and 'message')
     *
     */
    public void run(Map<String, String> args) throws URISyntaxException {
        EchoRequest request = new EchoRequest();

        request.setOinBasisregistratie(args.get("oin"));
        request.setEcho(args.get("message"));

        this.actionResult = this.annotatieClient.sendRequest(request);
    }

    @Override
    public ActionResult getResult() {
        return this.actionResult;
    }
}

