package nl.overheid.stelsel.digimelding.actions;

import nl.overheid.stelsel.digimelding.annotatie.StatusoverzichtRequest;
import nl.overheid.stelsel.digimelding.settings.SettingsManager;
import nl.overheid.stelsel.digimelding.clients.AnnotatieClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.net.URISyntaxException;
import java.util.Map;

/**
 * An action performing the 'StatusoverzichtRequest'.
 */
@Component
public class StatusOverzichtAction implements IAction {

    private AnnotatieClient annotatieClient;
    private SettingsManager settingsManager;
    private ActionResult actionResult;

    @Autowired
    public StatusOverzichtAction(AnnotatieClient annotatieClient, SettingsManager settingsManager) {
        this.annotatieClient = annotatieClient;
        this.settingsManager = settingsManager;
    }

    /**
     * Perform a StatusoverzichtRequest. Set the response on the ActionResult.
     *
     * @param args - he arguments of the request (should contains a key for 'oin')
     *
     */
    public void run(Map<String, String> args) throws URISyntaxException {
        StatusoverzichtRequest request = new StatusoverzichtRequest();

        request.setOinBasisregistratie(args.get("oin"));
        request.setOinVragendeOrganisatie(settingsManager.getOin());

        this.actionResult = this.annotatieClient.sendRequest(request);
    }

    @Override
    public ActionResult getResult() {
        return this.actionResult;
    }
}
