package nl.overheid.stelsel.digimelding.clients;

import nl.overheid.stelsel.digimelding.actions.ActionResult;
import org.springframework.ws.client.core.support.WebServiceGatewaySupport;
import org.springframework.ws.soap.addressing.client.ActionCallback;
import org.springframework.ws.soap.addressing.core.EndpointReference;
import org.springframework.ws.soap.addressing.messageid.UuidMessageIdStrategy;

import java.net.URI;
import java.net.URISyntaxException;

public abstract class AbstractClient extends WebServiceGatewaySupport {

    private static final String REPLY_TO_ADDRESS = "http://www.w3.org/2005/08/addressing/anonymous";

    public ActionResult sendRequest(Object request) throws URISyntaxException {
        ActionResult actionResult = new ActionResult();

        actionResult.setResponse(
                sendRequest(request, getCallback(request.getClass().getSimpleName())));

        return actionResult;
    }

    public Object sendRequest(Object request, ActionCallback callback) {
        return getWebServiceTemplate().marshalSendAndReceive(request, callback);
    }

    public abstract String getBaseUrl();

    /**
     * Creates a callback for requests.
     *
     * @param requestName - The name of the request to make the callback to.
     * @return The callback
     *
     * @throws URISyntaxException
     */
    private ActionCallback getCallback(String requestName) throws URISyntaxException {
        ActionCallback callback = new ActionCallback(this.getBaseUrl() + requestName);

        callback.setReplyTo(new EndpointReference(new URI(REPLY_TO_ADDRESS)));
        callback.setMessageIdStrategy(new UuidMessageIdStrategy());

        return callback;
    }
}
