package nl.overheid.stelsel.digimelding.clients;

import nl.overheid.stelsel.digimelding.interceptors.MessageInterceptor;
import nl.overheid.stelsel.digimelding.settings.SettingsManager;
import nl.overheid.stelsel.digimelding.interceptors.SecurityRequestInterceptor;
import nl.overheid.stelsel.digimelding.interceptors.SecurityResponseInterceptor;
import nl.overheid.stelsel.digimelding.marshallers.AnnotatieMarshaller;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.ws.client.support.interceptor.ClientInterceptor;

/**
 * Client for making Annotatie requests.
 */
@Component
public class AnnotatieClient extends AbstractClient {

    private static final String ANNOTATIE_ACTION_CALLBACK_BASE_URL = "http://www.digimelding.nl/dmks/cookiebox/";

    @Autowired
    public AnnotatieClient(SettingsManager settingsManager, AnnotatieMarshaller annotatieMarshaller,
                           MessageInterceptor messageInterceptor, SecurityRequestInterceptor securityRequestInterceptor,
                           SecurityResponseInterceptor securityResponseInterceptor) {
        setDefaultUri(settingsManager.getAnnotatieEndpoint());
        setMarshaller(annotatieMarshaller);
        setUnmarshaller(annotatieMarshaller);

        //Securityinterceptors provide, for example, a signature for a request and validation of the signature for the response
        setInterceptors(new ClientInterceptor[]{securityRequestInterceptor, securityResponseInterceptor, messageInterceptor});
    }

    public String getBaseUrl(){
        return ANNOTATIE_ACTION_CALLBACK_BASE_URL;
    }
}
