package nl.overheid.stelsel.digimelding.clients;

import nl.overheid.stelsel.digimelding.interceptors.MessageInterceptor;
import nl.overheid.stelsel.digimelding.settings.SettingsManager;
import nl.overheid.stelsel.digimelding.marshallers.CatalogusMarshaller;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.ws.client.support.interceptor.ClientInterceptor;

/**
 * Client for making Catalogus requests.
 *
 * Note the absance of Securityinterceptors, they are not used for these requests.
 */
@Component
public class CatalogusClient extends AbstractClient {

    private static final String CATALOGUS_ACTION_CALLBACK_BASE_URL = "http://webservices.digimelding.nl/catalogus/";

    @Autowired
    public CatalogusClient(SettingsManager settingsManager, CatalogusMarshaller catalogusMarshaller,
                           MessageInterceptor messageInterceptor) {
        setDefaultUri(settingsManager.getCatalogusEndpoint());
        setMarshaller(catalogusMarshaller);
        setUnmarshaller(catalogusMarshaller);
        setInterceptors(new ClientInterceptor[]{messageInterceptor});
    }

    public String getBaseUrl(){
        return CATALOGUS_ACTION_CALLBACK_BASE_URL;
    }
}
