package nl.overheid.stelsel.digimelding.commands;

import nl.overheid.stelsel.digimelding.actions.IAction;

import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.Map;
import java.util.function.BooleanSupplier;

/**
 * Command class for checking and executing actions.
 */
public class CommandRunner implements ICommand {
    private IAction action;
    private Map<String, String> args;
    private BooleanSupplier executable;

    /**
     * @param action - The action that is to be executed.
     * @param executable - Used to check whether the action can/should be executed.
     */
    public CommandRunner(IAction action, BooleanSupplier executable)
    {
        this.action = action;
        this.executable = executable;
        this.args = new HashMap<>();
    }

    public void execute() throws URISyntaxException {
        action.run(this.args);
    }

    public boolean canExecute()
    {
        return executable.getAsBoolean();
    }

    public void addArgument(String key, String value)
    {
        args.put(key, value);
    }

    public IAction getAction(){
        return this.action;
    }
}
