package nl.overheid.stelsel.digimelding.commands;

import nl.overheid.stelsel.digimelding.actions.IAction;
import org.junit.Test;
import org.mockito.Mockito;

import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.Map;
import java.util.function.BooleanSupplier;

import static junit.framework.TestCase.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

public class CommandRunnerTest {

    @Test
    public void testExecuteCommand() throws URISyntaxException {
        IAction actionMock = Mockito.mock(IAction.class);
        BooleanSupplier executableMock = Mockito.mock(BooleanSupplier.class);

        when(executableMock.getAsBoolean()).thenReturn(true);
        CommandRunner runner = new CommandRunner(actionMock, executableMock);
        runner.execute();

        verify(actionMock, times(1)).run(new HashMap<>());
    }

    @Test
    public void testExecuteCommandWithArgument() throws URISyntaxException {
        IAction actionMock = Mockito.mock(IAction.class);
        BooleanSupplier executableMock = Mockito.mock(BooleanSupplier.class);

        when(executableMock.getAsBoolean()).thenReturn(true);
        CommandRunner runner = new CommandRunner(actionMock, executableMock);
        runner.addArgument("test-key", "test-value");
        runner.execute();

        Map<String, String> expectedArgs = new HashMap<>();
        expectedArgs.put("test-key", "test-value");

        verify(actionMock, times(1)).run(expectedArgs);
    }

    @Test
    public void testCreateExecutableCommand(){
        IAction actionMock = Mockito.mock(IAction.class);
        BooleanSupplier executableMock = Mockito.mock(BooleanSupplier.class);

        when(executableMock.getAsBoolean()).thenReturn(true);
        CommandRunner runner = new CommandRunner(actionMock, executableMock);

        assertTrue(runner.canExecute());
        assertEquals(actionMock, runner.getAction());
    }

    @Test
    public void testCreateNonExecutableCommand(){
        IAction actionMock = Mockito.mock(IAction.class);
        BooleanSupplier executableMock = Mockito.mock(BooleanSupplier.class);

        when(executableMock.getAsBoolean()).thenReturn(false);
        CommandRunner runner = new CommandRunner(actionMock, executableMock);

        assertFalse(runner.canExecute());
        assertEquals(actionMock, runner.getAction());
    }
}
