package nl.overheid.stelsel.digimelding.interceptors;

import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;
import org.springframework.ws.context.MessageContext;
import org.springframework.ws.soap.SoapMessage;

import static org.junit.Assert.*;

public class MessageInterceptorIT extends Wss4jTestCase {
    private MessageInterceptor messageInterceptor;
    private MessageContext messageContextMock;

    private String prettyEmptyMessage =
            "<SOAP-ENV:Envelope xmlns:SOAP-ENV=\"http://schemas.xmlsoap.org/soap/envelope/\">\r\n" +
                    "    <SOAP-ENV:Body/>\r\n" +
                    "</SOAP-ENV:Envelope>\r\n";
    @Before
    public void setUp() throws Exception {
        super.setUp();
        this.messageContextMock = Mockito.mock(MessageContext.class);
        this.messageInterceptor = new MessageInterceptor();
    }
    @Test
    public void testHandleRequest() throws Exception {
        SoapMessage message = loadSaaj11Message("/empty-soap.xml");
        MessageContext messageContext = getSoap11MessageContext(message);

        assertTrue(messageInterceptor.handleRequest(messageContext));

        assertEquals(prettyEmptyMessage, this.messageInterceptor.getRequest());
    }

    @Test
    public void testHandleResponse() throws Exception {
        SoapMessage message = loadSaaj11Message("/empty-soap.xml");
        MessageContext messageContext = getSoap11MessageContext(message);

        assertTrue(messageInterceptor.handleResponse(messageContext));

        assertEquals(prettyEmptyMessage, this.messageInterceptor.getResponse());
    }

    @Test
    public void testHandleFault() throws Exception {
        SoapMessage message = loadSaaj11Message("/empty-soap.xml");
        MessageContext messageContext = getSoap11MessageContext(message);

        assertFalse(messageInterceptor.handleFault(messageContext));

        assertEquals(prettyEmptyMessage, this.messageInterceptor.getResponse());
    }

    @Test
    public void testClearRequestAndResponse() throws Exception {
        SoapMessage message = loadSaaj11Message("/empty-soap.xml");
        MessageContext messageContext = getSoap11MessageContext(message);

        messageInterceptor.handleRequest(messageContext);
        messageInterceptor.handleResponse(messageContext);

        assertNotNull(messageInterceptor.getRequest());
        assertNotNull(messageInterceptor.getResponse());

        messageInterceptor.clearRequestAndResponse();

        assertNull(messageInterceptor.getRequest());
        assertNull(messageInterceptor.getResponse());
    }
}
