package nl.overheid.stelsel.digimelding.interceptors;

import nl.overheid.stelsel.digimelding.exceptions.InvalidKeystoreSettingsException;
import nl.overheid.stelsel.digimelding.settings.SettingsManager;
import org.apache.wss4j.common.crypto.Crypto;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.mockito.Mockito;

import java.io.File;
import java.io.FileReader;
import java.util.Properties;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.*;

public class SecurityResponseInterceptorIT {

    private SecurityResponseInterceptor securityResponseInterceptor;
    private SecurityRequestInterceptor securityRequestInterceptorMock;

    private SettingsManager settingsManagerMock;

    @Before
    public void setUp() {
        settingsManagerMock = Mockito.mock(SettingsManager.class);
        securityRequestInterceptorMock = Mockito.mock(SecurityRequestInterceptor.class);
        when(settingsManagerMock.getClientCertificateAlias()).thenReturn("test");

        this.securityResponseInterceptor = new SecurityResponseInterceptor(settingsManagerMock);
    }

    @Test
    public void testSetSecumentWithEmptyKeystore() throws Exception {
        when(settingsManagerMock.getTrustStorePassword()).thenReturn("password");
        when(settingsManagerMock.getTrustStorePath()).thenReturn("truststore.p12");

        SecurityResponseInterceptor spy = Mockito.spy(this.securityResponseInterceptor);

        spy.setValidation();
        verify(spy, times(1)).setValidationSignatureCrypto(any(Crypto.class));
    }

    @Test
    public void testSetSecumentWithWrongPasswordForKeystoreThrowInvalidKeystoreSettingsException() {
        when(settingsManagerMock.getKeyStorePassword()).thenReturn("wrong-password");
        when(settingsManagerMock.getKeyStorePath()).thenReturn("keystore.p12");

        SecurityResponseInterceptor spy = Mockito.spy(this.securityResponseInterceptor);

        try {
            spy.setValidation();
            fail("Should have thrown exception");
        } catch(Exception e){
            assertEquals(e.getClass(), InvalidKeystoreSettingsException.class);
        }
        verify(spy, times(1)).setValidationSignatureCrypto(any(Crypto.class));
    }

    @Test
    public void testSetSecumentWithNonExistingKeystoreThrowInvalidKeystoreSettingsException() {
        when(settingsManagerMock.getKeyStorePassword()).thenReturn("password");
        when(settingsManagerMock.getKeyStorePath()).thenReturn("non-existing-keystore.p12");

        SecurityResponseInterceptor spy = Mockito.spy(this.securityResponseInterceptor);

        try {
            spy.setValidation();
            fail("Should have thrown exception");
        } catch(Exception e){
            assertEquals(e.getClass(), InvalidKeystoreSettingsException.class);
        }
        verify(spy, times(1)).setValidationSignatureCrypto(any(Crypto.class));
    }

}
