package nl.overheid.stelsel.digimelding.helpers;

import javafx.scene.control.Alert;
import javafx.scene.control.ButtonType;
import javafx.scene.control.Label;
import javafx.scene.control.TextArea;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.Priority;
import org.springframework.stereotype.Component;

import java.io.PrintWriter;
import java.io.StringWriter;

@Component
public class AlertHelper {
    public void showAlert(Alert.AlertType type, String title, String content){
        Alert alert = new Alert(type);
        alert.setTitle(title);
        alert.setHeaderText(null);
        alert.setContentText(content);
        alert.showAndWait();
    }

    public void showStackTraceAlert(Alert.AlertType type, String title, String content, Exception exception) {
        Alert alert = new Alert(type);
        alert.setTitle(title);
        alert.setHeaderText(null);
        alert.setContentText(content);

        alert.getDialogPane().setExpandableContent(getExpandableContent(exception));
        alert.showAndWait();
    }

    public void invalidSecuritySettingsAlert(Exception e, String title){
        showStackTraceAlert(
                Alert.AlertType.ERROR,
                title,
                "Er is ofwel geen geldige keystore/truststore gekozen, of een van de wachtwoorden is incorrect. \n\n" +
                        "Klik op 'Show Details' voor meer uitleg", e);
    }

    public void noConnectionPossibleAlert(Exception e){
        showStackTraceAlert(
                Alert.AlertType.ERROR,
                "Geen verbinding!",
                "Controleer uw firewallinstellingen en of uw toegang heeft tot Digimelding vanaf uw netwerk!", e
        );
    }

    public void invalidInformationAlert(Exception e){
        showStackTraceAlert(Alert.AlertType.INFORMATION, "Fout!",
                "De ontvangen fout is:\n\n" + e.getMessage() + "\n\n Zie de respons voor het volledige bericht.", e);
    }

    public void missingInformationAlert(String message){
        showAlert(Alert.AlertType.INFORMATION, "Niet verzonden!", message);
    }

    public void generalErrorAlert(Exception e){
        showStackTraceAlert(Alert.AlertType.ERROR, "Er ging iets mis", e.getMessage(), e);
    }

    public void generalErrorAlert(String message, Exception e){
        showStackTraceAlert(Alert.AlertType.ERROR, "Er ging iets mis", message, e);
    }

    public void showSuccesAlert(String message){
        Alert alert = new Alert(Alert.AlertType.INFORMATION, message, ButtonType.OK);
        alert.setHeaderText(null);
        alert.setTitle(null);
        alert.showAndWait();
    }

    private GridPane getExpandableContent(Exception exception){
        StringWriter sw = new StringWriter();
        PrintWriter pw = new PrintWriter(sw);
        exception.printStackTrace(pw);

        TextArea textArea = getExpandableTextArea(sw);

        GridPane.setVgrow(textArea, Priority.ALWAYS);
        GridPane.setHgrow(textArea, Priority.ALWAYS);

        GridPane expContent = new GridPane();
        expContent.setMaxWidth(Double.MAX_VALUE);
        expContent.add(new Label(), 0, 0);
        expContent.add(textArea, 0, 1);

        return expContent;
    }

    private TextArea getExpandableTextArea(StringWriter sw){
        TextArea textArea = new TextArea(sw.toString());
        textArea.setEditable(false);
        textArea.setWrapText(true);

        textArea.setMaxWidth(Double.MAX_VALUE);
        textArea.setMaxHeight(Double.MAX_VALUE);

        return textArea;
    }
}
