﻿using Core.Digimelding.Koppeling.Catalogus;
using Core.Digimelding.Koppeling.Clients;
using Core.Digimelding.Koppeling.Exceptions;
using System.Collections.Generic;

namespace Core.Digimelding.Koppeling.Actions
{
    public class CatalogusAction : IAction
    {
        //Should be loaded via dependency injection
        private ICatalogusClient client;

        public IResult result { get; set; }

        private string oin;

        public CatalogusAction(ICatalogusClient catalogusClient, AbstractResultFactory resultFactory)
        {
            this.result = resultFactory.getResultWrapper();
            this.client = catalogusClient;
        }

        /// <summary>
        ///     Executes the CatalogusAction by performing a request to Digimelding Webservice.
        /// </summary>
        /// <param name="args">The arguments sent with the request, in this case it should contain a value for key 'oin'</param>
        public void Run(Dictionary<string, string> args)
        {
            SetArguments(args);

            OpvragenResponse response = this.client.getInterface().Opvragen(new OpvragenRequest(
                new CatalogusRequest() { OinBasisregistratie = oin }
            ));

            result.SetResponse(response.CatalogusResponse);
        }

        /// <summary>
        /// Set the arguments for the action. Throws a MissingArgumentException if arguments are missing.
        /// </summary>
        /// <param name="args"></param>
        private void SetArguments(Dictionary<string, string> args)
        {
            try
            {
                oin = args["oin"];
            }
            catch (KeyNotFoundException e)
            {
                result.SetResponse(null);
                throw new MissingArgumentException("OIN should be provided", e);
            }
        }
    }
}
