﻿using Core.Digimelding.Koppeling.Annotatie;
using Core.Digimelding.Koppeling.Clients;
using Core.Digimelding.Koppeling.Exceptions;
using System;
using System.Collections.Generic;

namespace Core.Digimelding.Koppeling.Actions
{
    public class EchoAction : IAction
    {
        //Should be loaded via dependency injection
        private IAnnotatieClient client;
        public IResult result { get; set; }

        private string message;
        private string oin;

        public EchoAction(IAnnotatieClient annotatieClient, AbstractResultFactory resultFactory)
        {
            this.result = resultFactory.getResultWrapper();
            this.client = annotatieClient;
        }

        /// <summary>
        ///     Executes the CatalogusAction by performing a request to Digimelding Webservice.
        /// </summary>
        /// <param name="args">The arguments sent with the request, in this case it should contain a value for key 'oin' and key 'message'</param>
        public void Run(Dictionary<string, string> args)
        {
            SetArguments(args);
            
            EchoResponse1 response = this.client.getInterface().Echo(
                new EchoRequest1(
                    new EchoRequest()
                    {
                        Echo = message,
                        OinBasisregistratie = oin
                    }
                )
            );

            result.SetResponse(response.EchoResponse);
        }

        /// <summary>
        /// Set the arguments for the action. Throws a MissingArgumentException if arguments are missing.
        /// </summary>
        /// <param name="args"></param>
        private void SetArguments(Dictionary<string, string> args)
        {
            try
            {
                message = args["message"];
                oin = args["oin"];
            }
            catch (KeyNotFoundException e)
            {
                result.SetResponse(null);
                throw new MissingArgumentException("Both message and oin should be provided", e);
            }
        }
    }
}
