﻿using Core.Digimelding.Koppeling.Annotatie;
using Core.Digimelding.Koppeling.Clients;
using Core.Digimelding.Koppeling.Exceptions;
using System.Collections.Generic;

namespace Core.Digimelding.Koppeling.Actions
{
    public class StatusOverzichtAction : IAction
    {
        //Should be loaded via dependency injection
        private IAnnotatieClient client;
        private ISettingsManager settingsManager;

        public IResult result { get; set; }

        private string oin;

        public StatusOverzichtAction(IAnnotatieClient annotatieClient, ISettingsManager settingsManager, AbstractResultFactory resultFactory)
        {
            this.result = resultFactory.getResultWrapper();
            this.client = annotatieClient;
            this.settingsManager = settingsManager;

        }

        /// <summary>
        ///     Executes the CatalogusAction by performing a request to Digimelding Webservice.
        /// </summary>
        /// <param name="args">The arguments sent with the request, in this case it should contain a value for key 'oin'</param>
        public void Run(Dictionary<string, string> args)
        {
            SetArguments(args);

            StatusoverzichtResponse1 response = this.client.getInterface().Statusoverzicht(
                new StatusoverzichtRequest1(
                    new StatusoverzichtRequest()
                    {
                        OinVragendeOrganisatie = settingsManager.Oin,
                        OinBasisregistratie = oin
                    }
                )
            );

            result.SetResponse(response.StatusoverzichtResponse);
        }

        /// <summary>
        /// Set the arguments for the action. Throws a MissingArgumentException if arguments are missing.
        /// </summary>
        /// <param name="args"></param>
        private void SetArguments(Dictionary<string, string> args)
        {
            try
            {
                oin = args["oin"];
            }
            catch (KeyNotFoundException e)
            {
                result.SetResponse(null);
                throw new MissingArgumentException("OIN should be provided", e);
            }
        }
    }
}
