﻿using Core.Digimelding.Koppeling.Inspectors;
using System;
using System.Configuration;
using System.Net;
using System.ServiceModel.Channels;

namespace Core.Digimelding.Koppeling.Bindings
{
    /// <summary>
    /// Class for creating a CustomBinding with a custom elements.
    /// </summary>
    class BasicBinding : CustomBinding, IBinding
    {
        private static readonly long DEFAULT_BUFFERSIZE = 30000;

        public void setElements(IMessageInspector messageInspector)
        {
            addMtomBindingElement(messageInspector);
            addHttpsTransportBindingElement();
        }

        /// <summary>
        /// Add an element which makes sure messages are encoded and decoded correctly.
        /// </summary>
        /// <param name="messageInspector"></param>
        private void addMtomBindingElement(IMessageInspector messageInspector)
        {
            this.Elements.Add(new CustomMtomBindingElement(messageInspector));
        }

        /// <summary>
        /// Add an element with correct settings for HTTPS transport.
        /// </summary>
        private void addHttpsTransportBindingElement()
        {
            HttpsTransportBindingElement httpsTransportBindingElement = new HttpsTransportBindingElement
            {
                RequireClientCertificate = true,
                AuthenticationScheme = AuthenticationSchemes.Anonymous
            };

            long bufferSize = Convert.ToInt64(ConfigurationManager.AppSettings["max-buffer-size"]);

            //To avoid not being able to receive very large responses.
            httpsTransportBindingElement.MaxReceivedMessageSize = bufferSize > 0 ? bufferSize : DEFAULT_BUFFERSIZE;

            this.Elements.Add(httpsTransportBindingElement);
        }
    }
}
