﻿using Core.Digimelding.Koppeling.Inspectors;
using System.Configuration;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Security;
using System.ServiceModel.Security.Tokens;

namespace Core.Digimelding.Koppeling.Bindings
{
    /// <summary>
    /// Class for creating a CustomBinding with a custom elements.
    /// </summary>
    class SecureBinding : BasicBinding, IBinding
    {
        public new void setElements(IMessageInspector messageInspector)
        {
            addSecurityBindingElement();
            base.setElements(messageInspector);
        }

        /// <summary>
        /// Add an element with ensures the correct security settings are used.
        /// </summary>
        private void addSecurityBindingElement()
        {
            AsymmetricSecurityBindingElement securityBindingElement = SecurityBindingElement.CreateMutualCertificateDuplexBindingElement(
                MessageSecurityVersion.WSSecurity10WSTrustFebruary2005WSSecureConversationFebruary2005WSSecurityPolicy11BasicSecurityProfile10);
            securityBindingElement.SecurityHeaderLayout = SecurityHeaderLayout.LaxTimestampLast;
            securityBindingElement.AllowSerializedSigningTokenOnReply = true;
            securityBindingElement.EndpointSupportingTokenParameters.Signed.Add(new X509SecurityTokenParameters());
            securityBindingElement.EnableUnsecuredResponse = true;
            securityBindingElement.DefaultAlgorithmSuite = SecurityAlgorithmSuite.Basic256Sha256;

            this.Elements.Add(securityBindingElement);
        }
    }
}
