﻿using Core.Digimelding.Koppeling.Annotatie;
using Core.Digimelding.Koppeling.Bindings;
using Core.Digimelding.Koppeling.Inspectors;
using System;
using System.ServiceModel;

namespace Core.Digimelding.Koppeling.Clients
{
    /// <summary>
    ///     This client provides an interface for communication with the Digimelding Webservice.
    /// </summary>
    public class AnnotatieClient : Client<AnnotatieInterface>, IAnnotatieClient
    {
        public AnnotatieClient(ISettingsManager settingsManager, IMessageInspector messageInspector)
        {
            this.MessageInspector = messageInspector;
            this.SettingsManager = settingsManager;
        }

        /// <summary>
        ///     Creates a factory able to create an AnnotatieInterface. Makes sure the right security settings are used and references are made to the correct certificates.
        /// </summary>
        /// <returns>
        ///     ChannelFactory -Builds the AnnotatieInterface to interact with the Digimelding Webservice.
        /// </returns>
        public override ChannelFactory<AnnotatieInterface> CreateFactory()
        { 
            SecureBinding secureBinding = new SecureBinding();
            secureBinding.setElements(this.MessageInspector);

            EndpointAddress address = new EndpointAddress(new Uri(this.SettingsManager.AnnotatieEndpoint),
                EndpointIdentity.CreateDnsIdentity(this.SettingsManager.AnnotatieDns));

            ChannelFactory<AnnotatieInterface> factory = new ChannelFactory<AnnotatieInterface>(secureBinding, address);
           
            //Only sign and no encryption
            setProtectionLevel(factory);

            return factory;
        }

        public AnnotatieInterface getInterface()
        {
            return this.Channel;
        }
    }
}
