﻿using Core.Digimelding.Koppeling.Bindings;
using Core.Digimelding.Koppeling.Catalogus;
using Core.Digimelding.Koppeling.Inspectors;
using System;
using System.ServiceModel;

namespace Core.Digimelding.Koppeling.Clients
{
    /// <summary>
    ///     This client provides an interface for communication with the Digimelding Webservice.
    /// </summary>
    public class CatalogusClient : Client<CatalogusInterface>, ICatalogusClient
    {
        public CatalogusClient(ISettingsManager settingsManager, IMessageInspector messageInspector)
        {
            this.SettingsManager = settingsManager;
            this.MessageInspector = messageInspector;
        }

        /// <summary>
        ///     Creates a factory able to create an AnnotatieInterface. Makes sure the references are made to the correct certificates.
        /// </summary>
        /// <returns>
        ///     ChannelFactory - Builds the CatalogusInterface to interact with the Digimelding Webservice.
        /// </returns>
        public override ChannelFactory<CatalogusInterface> CreateFactory()
        {
            BasicBinding basicBinding = new BasicBinding();
            basicBinding.setElements(this.MessageInspector);

            EndpointAddress address = new EndpointAddress(new Uri(this.SettingsManager.CatalogusEndpoint),
                EndpointIdentity.CreateDnsIdentity(this.SettingsManager.CatalogusDns));

            ChannelFactory<CatalogusInterface> factory = new ChannelFactory<CatalogusInterface>(basicBinding, address);

            //Only sign and no encryption
            setProtectionLevel(factory);

            return factory;
        }

        public CatalogusInterface getInterface()
        {
            return this.Channel;
        }
    }
}
