﻿using Core.Digimelding.Koppeling.Inspectors;
using System;
using System.ServiceModel;

namespace Core.Digimelding.Koppeling.Clients
{
    /// <summary>
    /// Abstract class which serves as a base for clients communicating with the  Webservice.
    /// </summary>
    /// <typeparam name="TChannel"></typeparam>
    public abstract class Client<TChannel>
    {
        public IMessageInspector MessageInspector { get; set; }
        public ISettingsManager SettingsManager { get; set; }
        public TChannel Channel { get; private set; }

        /// <summary>
        ///     Creates a factory able to create an AnnotatieInterface. Makes sure the right security settings are used and references are made to the correct certificates.
        /// </summary>
        /// <returns>
        ///     ChannelFactory -Builds the AnnotatieInterface to interact with the Digimelding Webservice.
        /// </returns>
        public abstract ChannelFactory<TChannel> CreateFactory();

        public void CreateChannel()
        {
            ChannelFactory<TChannel> factory = this.CreateFactory();

            factory.Credentials.ClientCertificate.Certificate = this.SettingsManager.getClientCertificate();
            factory.Credentials.ServiceCertificate.DefaultCertificate = this.SettingsManager.getServerCertificate();

            TChannel channel = factory.CreateChannel();

            ICommunicationObject commObject = channel as ICommunicationObject;

            if (commObject != null)
            {
                commObject.Faulted += new EventHandler(ClientFaulted);
            }

            this.Channel = channel;
        }

        protected void setProtectionLevel(ChannelFactory<TChannel> factory)
        {
            factory.Endpoint.Contract.ProtectionLevel = System.Net.Security.ProtectionLevel.Sign;
        }

        private void ClientFaulted(object sender, EventArgs e)
        {
            CreateChannel();
        }
    }
}
