﻿using Core.Digimelding.Koppeling.Actions;
using System;
using System.Collections.Generic;

namespace Core.Digimelding.Koppeling.Commands
{
    /// <summary>
    /// Runner for executing a command delegating by running an action.
    /// </summary>
    public class CommandRunner : ICommand
    {
        private IAction action;
        private Func<bool> _canExecute;
        private Dictionary<string, string> args;

        public CommandRunner(IAction action, Func<bool> canExecute)
        {
            this.action = action;
            this._canExecute = canExecute;
            this.args = new Dictionary<string, string>();
        }

        /// <summary>
        ///     Executes the action with the provided arguments.
        /// </summary>
        public void Execute()
        {
             action.Run(this.args);
        }

        /// <summary>
        /// Checks if the action can be executed.
        /// </summary>
        /// <returns>True or False</returns>
        public bool canExecute()
        {
            return _canExecute();
        }

        /// <summary>
        /// Adds an argument to the list of arguments for the action.
        /// </summary>
        /// <param name="key"></param>
        /// <param name="value"></param>
        public void addArgument(string key, string value)
        {
            args.Add(key, value);
        }

        /// <summary>
        /// Returns the arguments for the action.
        /// </summary>
        /// <returns></returns>
        public Dictionary<string, string> getArguments()
        {
            return args;
        }
    }
}
