﻿using Core.Digimelding.Koppeling.Actions;
using Core.Digimelding.Koppeling.Annotatie;
using Core.Digimelding.Koppeling.Catalogus;
using Core.Digimelding.Koppeling.Clients;
using Core.Digimelding.Koppeling.Exceptions;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;
using System;
using System.Collections.Generic;

namespace ActionsTests
{
    [TestClass]
    public class CatalogusActionTests
    {
        private Mock<ICatalogusClient> catalogusClientMock;
        private Mock<AbstractResultFactory> abstractResultFactoryMock;
        private Mock<CatalogusInterface> catalogusInterfaceMock;
        private Mock<IResult> actionResultMock;

        [TestInitialize()]
        public void Initialize()
        {
            catalogusClientMock = new Mock<ICatalogusClient>();
            abstractResultFactoryMock = new Mock<AbstractResultFactory>();
            actionResultMock = new Mock<IResult>();
            catalogusInterfaceMock = new Mock<CatalogusInterface>();

            abstractResultFactoryMock
                .Setup(m => m.getResultWrapper())
                .Returns(actionResultMock.Object);
        }

        [TestMethod]
        public void TestCreateCatalogusAction()
        {
            CatalogusAction catalogusAction = new CatalogusAction(catalogusClientMock.Object, abstractResultFactoryMock.Object);
            
            Assert.AreEqual(catalogusAction.result, actionResultMock.Object);
        }

        [TestMethod]
        public void TestRunCatalogusAction()
        {
            catalogusClientMock
                .SetupSequence(m => m.getInterface())
                .Returns(catalogusInterfaceMock.Object);

            CatalogusAction catalogusAction = new CatalogusAction(catalogusClientMock.Object, abstractResultFactoryMock.Object);

            OpvragenResponse response = new OpvragenResponse();

            catalogusInterfaceMock
               .SetupSequence(m => m.Opvragen(It.IsAny<OpvragenRequest>()))
               .Returns(response);

            Dictionary<string, string> testArguments = new Dictionary<string, string>()
            {
                { "oin", "01234567890123456789"}
            };

            catalogusAction.Run(testArguments);

            catalogusClientMock.Verify(m => m.getInterface(), Times.Once);
            actionResultMock.Verify(m => m.SetResponse(response.CatalogusResponse), Times.Once);
        }

        [TestMethod]
        public void TestRunCatalogusActionWithMissingParamaterThrowError()
        {
            catalogusClientMock
                .SetupSequence(m => m.getInterface())
                .Returns(catalogusInterfaceMock.Object);

            CatalogusAction catalogusAction = new CatalogusAction(catalogusClientMock.Object, abstractResultFactoryMock.Object);

            Dictionary<string, string> testArguments = new Dictionary<string, string>()
                {
                    { "non-existing-key", "01234567890123456789"}
                };

            try
            {
                catalogusAction.Run(testArguments);
                Assert.Fail();
            }
            catch (Exception e)
            {
                Assert.IsInstanceOfType(e, typeof(MissingArgumentException));
                catalogusClientMock.Verify(m => m.getInterface(), Times.Never);
                catalogusInterfaceMock.Verify(m => m.Opvragen(It.IsAny<OpvragenRequest>()), Times.Never);
                actionResultMock.Verify(m => m.SetResponse(null), Times.Once);
                actionResultMock.Verify(m => m.SetResponse(It.IsAny<Object>()), Times.Once);
            }
        }
    }
}