﻿using Core.Digimelding.Koppeling.Actions;
using Core.Digimelding.Koppeling.Annotatie;
using Core.Digimelding.Koppeling.Clients;
using Core.Digimelding.Koppeling.Exceptions;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;
using System;
using System.Collections.Generic;

namespace ActionsTests
{ 
    [TestClass]
    public class EchoActionTests
    {
        private Mock<IAnnotatieClient> annotatieClientMock;
        private Mock<AbstractResultFactory> abstractResultFactoryMock;
        private Mock<AnnotatieInterface> annotatieInterfaceMock;
        private Mock<IResult> actionResultMock;

        [TestInitialize()]
        public void Initialize()
        {
            annotatieClientMock = new Mock<IAnnotatieClient>();
            abstractResultFactoryMock = new Mock<AbstractResultFactory>();
            actionResultMock = new Mock<IResult>();
            annotatieInterfaceMock = new Mock<AnnotatieInterface>();

            abstractResultFactoryMock
                .Setup(m => m.getResultWrapper())
                .Returns(actionResultMock.Object);
        }

        [TestMethod]
        public void TestCreateEchoAction()
        {
            EchoAction echoAction = new EchoAction(annotatieClientMock.Object, abstractResultFactoryMock.Object);

            Assert.AreEqual(echoAction.result, actionResultMock.Object);
        }

        [TestMethod]
        public void TestRunEchoAction()
        {
            annotatieClientMock
                .SetupSequence(m => m.getInterface())
                .Returns(annotatieInterfaceMock.Object);

            EchoAction echoAction = new EchoAction(annotatieClientMock.Object, abstractResultFactoryMock.Object);

            EchoResponse1 response = new EchoResponse1();
          
            annotatieInterfaceMock
               .SetupSequence(m => m.Echo(It.IsAny<EchoRequest1>()))
               .Returns(response);

            Dictionary<string, string> testArguments = new Dictionary<string, string>()
            {
                { "message", "test_message"},
                { "oin", "01234567890123456789"}
            };

            echoAction.Run(testArguments);

            annotatieClientMock.Verify(m => m.getInterface(), Times.Once);
            actionResultMock.Verify(m => m.SetResponse(response.EchoResponse), Times.Once);
        }

        [TestMethod]
        public void TestRunEchoActionWithMissingParamaterThrowError()
        {
            annotatieClientMock
                .SetupSequence(m => m.getInterface())
                .Returns(annotatieInterfaceMock.Object);

            EchoAction echoAction = new EchoAction(annotatieClientMock.Object, abstractResultFactoryMock.Object);

            Dictionary<string, string> testArguments = new Dictionary<string, string>()
                {
                    { "oin", "01234567890123456789"}
                };

            try
            {
                echoAction.Run(testArguments);
                Assert.Fail();
            } catch(Exception e)
            {
                Assert.IsInstanceOfType(e, typeof(MissingArgumentException));
                annotatieClientMock.Verify(m => m.getInterface(), Times.Never);
                annotatieInterfaceMock.Verify(m => m.Echo(It.IsAny<EchoRequest1>()), Times.Never);
                actionResultMock.Verify(m => m.SetResponse(null), Times.Once);
                actionResultMock.Verify(m => m.SetResponse(It.IsAny<Object>()), Times.Once);
            }
        }
    }
}