﻿using Core.Digimelding.Koppeling;
using Core.Digimelding.Koppeling.Actions;
using Core.Digimelding.Koppeling.Annotatie;
using Core.Digimelding.Koppeling.Clients;
using Core.Digimelding.Koppeling.Exceptions;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;
using System;
using System.Collections.Generic;

namespace ActionsTests
{
    [TestClass]
    public class StatusOverzichtActionTests
    {
        private Mock<IAnnotatieClient> annotatieClientMock;
        private Mock<AbstractResultFactory> abstractResultFactoryMock;
        private Mock<AnnotatieInterface> annotatieInterfaceMock;
        private Mock<IResult> actionResultMock;
        private Mock<ISettingsManager> settingsManagerMock;

        [TestInitialize()]
        public void Initialize()
        {
            annotatieClientMock = new Mock<IAnnotatieClient>();
            abstractResultFactoryMock = new Mock<AbstractResultFactory>();
            actionResultMock = new Mock<IResult>();
            annotatieInterfaceMock = new Mock<AnnotatieInterface>();
            settingsManagerMock = new Mock<ISettingsManager>();

            abstractResultFactoryMock
                .Setup(m => m.getResultWrapper())
                .Returns(actionResultMock.Object);

            settingsManagerMock
                .SetupSequence(m => m.Oin)
                .Returns("01234567891011121314");
        }

        [TestMethod]
        public void TestCreateStatusOverzichtAction()
        {
            StatusOverzichtAction echoAction = new StatusOverzichtAction(annotatieClientMock.Object, settingsManagerMock.Object, 
                abstractResultFactoryMock.Object);
            
            Assert.AreEqual(echoAction.result, actionResultMock.Object);
        }

        [TestMethod]
        public void TestRunStatusOverzichtAction()
        {
            annotatieClientMock
                .SetupSequence(m => m.getInterface())
                .Returns(annotatieInterfaceMock.Object);

            StatusOverzichtAction statusOverzichtAction = new StatusOverzichtAction(annotatieClientMock.Object, 
                settingsManagerMock.Object, abstractResultFactoryMock.Object);

            StatusoverzichtResponse1 response = new StatusoverzichtResponse1();

            annotatieInterfaceMock
               .SetupSequence(m => m.Statusoverzicht(It.IsAny<StatusoverzichtRequest1>()))
               .Returns(response);

            Dictionary<string, string> testArguments = new Dictionary<string, string>()
            {
                { "oin", "01234567890123456789"}
            };

            statusOverzichtAction.Run(testArguments);

            annotatieClientMock.Verify(m => m.getInterface(), Times.Once);
            actionResultMock.Verify(m => m.SetResponse(response.StatusoverzichtResponse), Times.Once);
        }

        [TestMethod]
        public void TestRunStatusOverzichtActionWithMissingParamaterThrowError()
        {
            annotatieClientMock
                .SetupSequence(m => m.getInterface())
                .Returns(annotatieInterfaceMock.Object);

            StatusOverzichtAction statusOverzichtAction = new StatusOverzichtAction(annotatieClientMock.Object, 
                settingsManagerMock.Object, abstractResultFactoryMock.Object);

            Dictionary<string, string> testArguments = new Dictionary<string, string>()
                {
                    { "non-existing-parameter", "01234567890123456789"}
                };

            try
            {
                statusOverzichtAction.Run(testArguments);
                Assert.Fail();
            }
            catch (Exception e)
            {
                Assert.IsInstanceOfType(e, typeof(MissingArgumentException));
                annotatieClientMock.Verify(m => m.getInterface(), Times.Never);
                annotatieInterfaceMock.Verify(m => m.Echo(It.IsAny<EchoRequest1>()), Times.Never);
                actionResultMock.Verify(m => m.SetResponse(null), Times.Once);
                actionResultMock.Verify(m => m.SetResponse(It.IsAny<Object>()), Times.Once);
            }
        }
    }
}