﻿using Core.Digimelding.Koppeling.Annotatie;
using Core.Digimelding.Koppeling.Clients;
using Core.Digimelding.Koppeling.Inspectors;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;
using System.ServiceModel;

namespace Core.Digimelding.Koppeling.Tests.ClientsTests
{
    [TestClass]
    public class AnnotatieClientTests
    {
        private Mock<ISettingsManager> settingsManagerMock;
        private Mock<IMessageInspector> messageInspectorMock;

        [TestInitialize()]
        public void Initialize()
        {
            settingsManagerMock = new Mock<ISettingsManager>();
            messageInspectorMock = new Mock<IMessageInspector>();
        }


        [TestMethod]
        public void TestCreateAnnotatieClient()
        {
            AnnotatieClient client = new AnnotatieClient(settingsManagerMock.Object, messageInspectorMock.Object);

            Assert.AreEqual(client.SettingsManager, settingsManagerMock.Object);
            Assert.AreEqual(client.MessageInspector, messageInspectorMock.Object);
        }

        [TestMethod]
        public void TestCreateFactory()
        {
            settingsManagerMock
                .Setup(m => m.AnnotatieDns)
                .Returns("test.digimelding.nl");

            settingsManagerMock
                .Setup(m => m.AnnotatieEndpoint)
                .Returns("https://test.digimelding.nl/api/koppelvlak");

            AnnotatieClient client = new AnnotatieClient(settingsManagerMock.Object, messageInspectorMock.Object);

            ChannelFactory<AnnotatieInterface> factory = client.CreateFactory();
            Assert.AreEqual(factory.Endpoint.Contract.ProtectionLevel, System.Net.Security.ProtectionLevel.Sign);
            Assert.AreEqual(factory.Endpoint.Address.ToString(), "https://test.digimelding.nl/api/koppelvlak");
        }
    }
}
