﻿using Core.Digimelding.Koppeling.Annotatie;
using Core.Digimelding.Koppeling.Catalogus;
using Core.Digimelding.Koppeling.Clients;
using Core.Digimelding.Koppeling.Inspectors;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;
using System.ServiceModel;

namespace Core.Digimelding.Koppeling.Tests.ClientsTests
{
    [TestClass]
    public class CatalogusClientTests
    {
        private Mock<ISettingsManager> settingsManagerMock;
        private Mock<IMessageInspector> messageInspectorMock;

        [TestInitialize()]
        public void Initialize()
        {
            settingsManagerMock = new Mock<ISettingsManager>();
            messageInspectorMock = new Mock<IMessageInspector>();
        }


        [TestMethod]
        public void TestCreateCatalogusClient()
        {
            CatalogusClient client = new CatalogusClient(settingsManagerMock.Object, messageInspectorMock.Object);

            Assert.AreEqual(client.SettingsManager, settingsManagerMock.Object);
            Assert.AreEqual(client.MessageInspector, messageInspectorMock.Object);
        }

        [TestMethod]
        public void TestCreateCatalogusFactory()
        {
            settingsManagerMock
                .Setup(m => m.CatalogusDns)
                .Returns("test.digimelding.nl");

            settingsManagerMock
                .Setup(m => m.CatalogusEndpoint)
                .Returns("https://test.digimelding.nl/api/catalogus");

            CatalogusClient client = new CatalogusClient(settingsManagerMock.Object, messageInspectorMock.Object);

            ChannelFactory<CatalogusInterface> factory = client.CreateFactory();
            Assert.AreEqual(factory.Endpoint.Contract.ProtectionLevel, System.Net.Security.ProtectionLevel.Sign);
            Assert.AreEqual(factory.Endpoint.Address.ToString(), "https://test.digimelding.nl/api/catalogus");
        }
    }
}
