﻿using Core.Digimelding.Koppeling.Actions;
using Core.Digimelding.Koppeling.Commands;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;
using System.Collections.Generic;

namespace Core.Digimelding.Koppeling.Tests.CommandsTests
{
    [TestClass]
    public class CommandRunnerTests
    {
        private Mock<IAction> actionMock;

        [TestInitialize()]
        public void Initialize()
        {
            actionMock = new Mock<IAction>();
        }

        [TestMethod]
        public void TestRunAction()
        {
            CommandRunner runner = new CommandRunner(actionMock.Object, canExecute);

            runner.Execute();

            actionMock.Verify(m => m.Run(It.IsAny<Dictionary<string, string>>()), Times.Once);

        }

        [TestMethod]
        public void TestRunActionWithArguments()
        {
            CommandRunner runner = new CommandRunner(actionMock.Object, canExecute);
            runner.addArgument("key", "value");
            runner.addArgument("key2", "value2");

            Dictionary<string, string> args = runner.getArguments();

            runner.Execute();

            actionMock.Verify(m => m.Run(args), Times.Once);
            Assert.AreEqual(args["key"], "value");
            Assert.AreEqual(args["key2"], "value2");
        }

        [TestMethod]
        public void TestCanExecuteWhenTrue()
        {
            CommandRunner runner = new CommandRunner(actionMock.Object, canExecute);
            Assert.IsTrue(runner.canExecute());
        }


        [TestMethod]
        public void TestCanExecuteWhenFalse()
        {
            CommandRunner runner = new CommandRunner(actionMock.Object, cannotExecute);
            Assert.IsFalse(runner.canExecute());
        }

        private bool canExecute()
        {
            return true;
        }

        private bool cannotExecute()
        {
            return false;
        }
    }
}
