﻿using Autofac;
using System.Windows;
using Voorbeeldkoppeling.Clients;
using Voorbeeldkoppeling.Windows;

namespace Voorbeeldkoppeling
{
    /// <summary>
    /// The starting point of the application. 
    /// 
    /// Makes sure the right dependencies are injected before starting up by loading the Main Window.
    /// </summary>
    public partial class App : Application
    {
        void App_Startup(object sender, StartupEventArgs e)
        {
            //Perform dependency injection
            var container = ContainerConfig.Configure();
            using (var scope = container.BeginLifetimeScope())
            {
                var app = scope.Resolve<IApplication>();
                app.Run(scope.Resolve<LoginWindow>());
            }
        }
    }

    public interface IApplication
    {
        void Run(LoginWindow window);
    }

    public class Program : IApplication
    {
        public void Run(LoginWindow window)
        {
            window.Show();
        }
    }

}
