﻿using Autofac;
using Core.Digimelding.Koppeling;
using Core.Digimelding.Koppeling.Actions;
using Core.Digimelding.Koppeling.Clients;
using Core.Digimelding.Koppeling.Inspectors;
using System.Configuration;
using Voorbeeldkoppeling.Controls;
using Voorbeeldkoppeling.Windows;

namespace Voorbeeldkoppeling.Clients
{
    /// <summary>
    /// Class which offers functionality to register components to be used for dependency injection.
    /// </summary>
    public static class ContainerConfig
    {
        public static IContainer Configure()
        {
            ContainerBuilder builder = new ContainerBuilder();

            builder.RegisterType<Program>().As<IApplication>().SingleInstance();

            ISettingsManager settingsManager = BuildSettingsManager();
            
            builder.RegisterInstance(settingsManager).As<ISettingsManager>();

            builder.RegisterType<AnnotatieClient>().As<IAnnotatieClient>().SingleInstance();
            builder.RegisterType<CatalogusClient>().As<ICatalogusClient>().SingleInstance();

            builder.RegisterType<EchoAction>().PropertiesAutowired().SingleInstance();
            builder.RegisterType<StatusOverzichtAction>().PropertiesAutowired().SingleInstance();
            builder.RegisterType<CatalogusAction>().PropertiesAutowired().SingleInstance();

            builder.RegisterType<SoapMessageInspector>().As<IMessageInspector>().SingleInstance();
            builder.RegisterType<ActionResultFactory>().As<AbstractResultFactory>().SingleInstance();

            builder.RegisterType<RequestControl>().PropertiesAutowired();
            builder.RegisterType<EchoControl>().PropertiesAutowired().SingleInstance();
            builder.RegisterType<StatusOverzichtControl>().PropertiesAutowired().SingleInstance();
            builder.RegisterType<CatalogusControl>().PropertiesAutowired().SingleInstance();
            builder.RegisterType<SettingsControl>().SingleInstance();

            builder.RegisterType<MainWindow>().SingleInstance();
            builder.RegisterType<LoginWindow>();

            return builder.Build();
        }

        private static ISettingsManager BuildSettingsManager()
        {
            ISettingsManager settingsManager = new SettingsManager();
       
            settingsManager.ServerCertificatePath = (string) Properties.Settings.Default.ServerCertificatePath;
            settingsManager.ClientCertificatePath = (string) Properties.Settings.Default.ClientCertificatePath;

            settingsManager.AnnotatieEndpoint = ConfigurationManager.AppSettings["digimelding.annotatie-endpoint-url"];
            settingsManager.AnnotatieDns = ConfigurationManager.AppSettings["digimelding.annotatie-endpoint-dns"];
            settingsManager.CatalogusEndpoint = ConfigurationManager.AppSettings["digimelding.catalogus-endpoint-url"];
            settingsManager.CatalogusDns = ConfigurationManager.AppSettings["digimelding.catalogus-endpoint-dns"];

            return settingsManager;
        }
    }
}
