﻿using Core.Digimelding.Koppeling;
using Core.Digimelding.Koppeling.Clients;
using System;
using System.Drawing;
using System.Security.Cryptography;
using System.Windows;
using Voorbeeldkoppeling.Dialogs;

namespace Voorbeeldkoppeling.Windows
{
    public partial class LoginWindow : Window
    {
        private static readonly log4net.ILog log = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        private ISettingsManager settingsManager;
        private MainWindow window;
        private IAnnotatieClient annotatieClient;
        private ICatalogusClient catalogusClient;

        public LoginWindow(ISettingsManager settingsManager, MainWindow window, IAnnotatieClient annotatieClient, ICatalogusClient catalogusClient)
        {
            InitializeComponent();

            this.settingsManager = settingsManager;
            this.window = window;
            this.annotatieClient = annotatieClient;
            this.catalogusClient = catalogusClient;

            window.SettingsControl.setSettingsWindow(this);

            this.ClientCertificatePath.Text = settingsManager.ClientCertificatePath;
            this.ServerCertificatePath.Text = settingsManager.ServerCertificatePath;
        }

        private void ClientCertificate_Click(object sender, RoutedEventArgs e)
        {
            Microsoft.Win32.OpenFileDialog dlg = new Microsoft.Win32.OpenFileDialog();

            dlg.DefaultExt = ".p12";
            dlg.Filter = "Certificates (*.p12)|*.p12";

            Nullable<bool> result = dlg.ShowDialog();
            
            if (result == true)
            {
                this.ClientCertificatePath.Text = dlg.FileName;
            }
        }

        private void ServerCertificate_Click(object sender, RoutedEventArgs e)
        {
            Microsoft.Win32.OpenFileDialog dlg = new Microsoft.Win32.OpenFileDialog();

            dlg.DefaultExt = ".cer";
            dlg.Filter = "Certificates (*.cer)|*.cer";

            Nullable<bool> result = dlg.ShowDialog();

            if (result == true)
            {
                this.ServerCertificatePath.Text = dlg.FileName;
            }
        }

        private void logIn(object sender, RoutedEventArgs e)
        {
            if (!isValid())
            {
                MessageBox.Show("Vul alle velden in!", "Inloggen mislukt!", MessageBoxButton.OK, MessageBoxImage.Information);
            }
            else
            {
                try
                {
                    setClientCertificate();
                    setServerCertificate();

                    Properties.Settings.Default.ClientCertificatePath = this.ClientCertificatePath.Text;
                    Properties.Settings.Default.ServerCertificatePath = this.ServerCertificatePath.Text;
                    Properties.Settings.Default.Save();

                    annotatieClient.CreateChannel();
                    catalogusClient.CreateChannel();

                    window.Show();
                    this.Visibility = Visibility.Hidden;
                }
                catch (Exception ce)
                {
                    log.Error("Het inloggen is mislukt: \n" + ce.ToString());
                }
            }
        }

        private bool isValid()
        {
            return !string.IsNullOrEmpty(this.ClientCertificatePath.Text) && !string.IsNullOrEmpty(this.ServerCertificatePath.Text) 
                && !string.IsNullOrEmpty(this.ClientCertificatePassword.Password);
        }

        private void setClientCertificate()
        {
            try {
                settingsManager.setClientCertificate(this.ClientCertificatePath.Text, this.ClientCertificatePassword.Password);
            }  catch (CryptographicException ce) {
                new StacktraceDialog("Instellingen niet correct!", "Er is ofwel geen geldig clientcertificaat gekozen, of het wachtwoord is niet correct.\n\n" +
                    "De foutmelding was: \n\n" + ce.Message, ce.ToString(), SystemIcons.Error);
                throw ce;
            } catch(InvalidOperationException ioe) {
                new StacktraceDialog("Instellingen niet correct!", "Er is geen geldig clientcertificaat gekozen.\n\n" +
                    "De foutmelding was: \n\n" + ioe.Message, ioe.ToString(), SystemIcons.Error);
                throw ioe;
            }
        }

        private void setServerCertificate()
        {
            try
            {
                settingsManager.setServerCertificate(this.ServerCertificatePath.Text);
            }
            catch (CryptographicException ce)
            {
                new StacktraceDialog("Instellingen niet correct!", "Er is geen geldig servercertificaat gekozen.\n\n" +
                    "De foutmelding was: \n\n" + ce.Message, ce.ToString(), SystemIcons.Error);
                throw ce;
            }
        }
    }
}
