﻿using System;
using System.Text;
using System.ServiceModel.Channels;
using System.IO;
using System.Xml;

namespace DigimeldingImplementatieEcho
{
	public class CustomMtomBindingElement : MessageEncodingBindingElement
	{
		MessageVersion _messageVersion = MessageVersion.Soap11WSAddressing10;

		public override MessageEncoderFactory CreateMessageEncoderFactory()
		{
			return new CustomMtomEncoderFactory(this._messageVersion);
		}
		public override MessageVersion MessageVersion
		{
			get
			{
				return this._messageVersion;
			}
			set
			{
				this._messageVersion = value;
			}
		}
		public override BindingElement Clone()
		{
			return this;
		}
		public override IChannelFactory<TChannel> BuildChannelFactory<TChannel>(BindingContext context)
		{
			context.BindingParameters.Add(this);
			return context.BuildInnerChannelFactory<TChannel>();
		}
		public override IChannelListener<TChannel> BuildChannelListener<TChannel>(BindingContext context)
		{
			context.BindingParameters.Add(this);
			return context.BuildInnerChannelListener<TChannel>();
		}
		public override bool CanBuildChannelFactory<TChannel>(BindingContext context)
		{
			return context.CanBuildInnerChannelFactory<TChannel>();
		}
		public override bool CanBuildChannelListener<TChannel>(BindingContext context)
		{
			return context.CanBuildInnerChannelListener<TChannel>();
		}
	}
	class CustomMtomEncoderFactory : MessageEncoderFactory
	{
		MessageVersion messageVersion;
		CustomMtomEncoder encoder;

		public CustomMtomEncoderFactory(MessageVersion messageVersion)
		{
			this.messageVersion = messageVersion;
			this.encoder = new CustomMtomEncoder(messageVersion);
		}
		public override MessageEncoder Encoder
		{
			get
			{
				return this.encoder;
			}
		}
		public override MessageVersion MessageVersion
		{
			get
			{
				return this.encoder.MessageVersion;
			}
		}
	}
	class CustomMtomEncoder : MessageEncoder
	{
		MessageEncoder _textEncoder;
		MessageEncoder _mtomEncoder;
		public CustomMtomEncoder(MessageVersion messageVersion)
		{
			_textEncoder = new TextMessageEncodingBindingElement(messageVersion, Encoding.UTF8).CreateMessageEncoderFactory().Encoder;
			_mtomEncoder = new MtomMessageEncodingBindingElement(messageVersion, Encoding.UTF8).CreateMessageEncoderFactory().Encoder;
		}
		public override string ContentType
		{
			get
			{
				return _textEncoder.ContentType;
			}
		}
		public override string MediaType
		{
			get
			{
				return _textEncoder.MediaType;
			}
		}
		public override MessageVersion MessageVersion
		{
			get
			{
				return _textEncoder.MessageVersion;
			}
		}
		public override bool IsContentTypeSupported(string contentType)
		{
			return this._mtomEncoder.IsContentTypeSupported(contentType);
		}
		public override T GetProperty<T>()
		{
			T result = this._textEncoder.GetProperty<T>();
			if (result == null)
			{
				result = this._mtomEncoder.GetProperty<T>();
			}
			if (result == null)
			{
				result = base.GetProperty<T>();
			}
			return result;
		}
		public override Message ReadMessage(ArraySegment<byte> buffer, BufferManager bufferManager, string contentType)
		{
			Message result = this._mtomEncoder.ReadMessage(buffer, bufferManager, contentType);
			return result;
		}
		public override ArraySegment<byte> WriteMessage(Message message, int maxMessageSize, BufferManager bufferManager, int messageOffset)
		{
			//if (false)
			//{
			//	using (MemoryStream ms = new MemoryStream())
			//	{
			//		XmlDictionaryWriter mtomWriter = XmlDictionaryWriter.CreateMtomWriter(ms, Encoding.UTF8, int.MaxValue, "text/xml", null, null, true, true);  //CreateMtomWriter(ms, message);
			//		message.WriteMessage(mtomWriter);
			//		mtomWriter.Flush();
			//		byte[] buffer = bufferManager.TakeBuffer((int)ms.Position + messageOffset);
			//		Array.Copy(ms.GetBuffer(), 0, buffer, messageOffset, (int)ms.Position);
			//		return new ArraySegment<byte>(buffer, messageOffset, (int)ms.Position);
			//	}
			//}
			//else
			//{
				return this._textEncoder.WriteMessage(message, maxMessageSize, bufferManager, messageOffset);
			//}
		}
		private static bool IsMtomMessage(string contentType)
		{
			return contentType.IndexOf("type=\"application/xop+xml\"", StringComparison.OrdinalIgnoreCase) >= 0;
		}
		private bool ShouldWriteMtom(Message message)
		{
			return false;
		}
		private XmlDictionaryWriter CreateMtomWriter(Stream stream, Message message)
		{
			return XmlDictionaryWriter.CreateMtomWriter(stream, Encoding.UTF8, int.MaxValue, "text/xml", null, "", true, true);
		}

		public override Message ReadMessage(Stream stream, int maxSizeOfHeaders, string contentType)
		{
			Message result = this._mtomEncoder.ReadMessage(stream, maxSizeOfHeaders, contentType);
			return result;
		}

		public override void WriteMessage(Message message, Stream stream)
		{
			//if (ShouldWriteMtom(message))
			//{
			//	XmlDictionaryWriter mtomWriter = CreateMtomWriter(stream, message);
			//	message.WriteMessage(mtomWriter);
			//}
			//else
			//{
				_textEncoder.WriteMessage(message, stream);
			//}
		}
	}
}
