﻿using System.Net;
using System.Security.Cryptography.X509Certificates;
using System.ServiceModel;
using System.ServiceModel.Channels;
using DigimeldingImplementatieEcho.Digimelding;

namespace DigimeldingImplementatieEcho
{
    class Program
    {
		static void Main(string[] args)
        {
			var client = CreateClient();
			var response = client.Echo(new EchoRequest1(new EchoRequest() { Echo = "test", OinBasisregistratie = "00000001006033404000" }));
			System.Diagnostics.Debug.WriteLine(response.EchoResponse.Echo);
		}

		public static AnnotatieInterface CreateClient()
        {
			var securityBindingElement = SecurityBindingElement.CreateMutualCertificateDuplexBindingElement(MessageSecurityVersion.WSSecurity10WSTrustFebruary2005WSSecureConversationFebruary2005WSSecurityPolicy11BasicSecurityProfile10);
	        securityBindingElement.SecurityHeaderLayout = SecurityHeaderLayout.LaxTimestampLast;
			securityBindingElement.AllowSerializedSigningTokenOnReply = true;

	        var customBinding = new CustomBinding();

	        // use a custom mtom bindingelement to send TextMessageEncoding, but receive MTOM
			var customMtomBindingElement = new CustomMtomBindingElement();

			// Create binding element for transport
			var httpsTransportBindingElement = new HttpsTransportBindingElement
			{
				RequireClientCertificate = true,
				AuthenticationScheme = AuthenticationSchemes.Anonymous
			};

	        customBinding.Elements.Add(securityBindingElement);
			customBinding.Elements.Add(customMtomBindingElement);
			customBinding.Elements.Add(httpsTransportBindingElement);

			var address = new EndpointAddress("https://acc.portaal.digimelding.nl/api/koppelvlak");

            var factory = new ChannelFactory<AnnotatieInterface>(customBinding, address);

	        factory.Credentials.ClientCertificate.SetCertificate(StoreLocation.LocalMachine, StoreName.My, X509FindType.FindBySubjectName, "<client pkio SSL certificaat>");
			factory.Credentials.ServiceCertificate.SetDefaultCertificate(StoreLocation.LocalMachine, StoreName.My, X509FindType.FindBySubjectName, "acc.portaal.digimelding.nl");


			return factory.CreateChannel();
        }
    }
}
